<?php

namespace Drupal\tmgmt_thebigword\Controller;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\tmgmt\Entity\JobItem;
use Drupal\tmgmt\Entity\RemoteMapping;
use Drupal\tmgmt\Entity\Translator;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt_thebigword\Plugin\tmgmt\Translator\ThebigwordTranslator;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Route controller of the remote callbacks for the tmgmt_thebigword module.
 */
class RemoteCallbackController extends ControllerBase {

  /**
   * Handles the notifications of changes in the files states.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request to handle.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response to return.
   */
  public function callback(Request $request) {
    $config = \Drupal::configFactory()->get('tmgmt_thebigword.settings');
    if ($config->get('debug')) {
      \Drupal::logger('tmgmt_thebigword')->debug('Request received %request.', ['%request' => $request]);
    }
    $project_id = $request->get('ProjectId');
    $file_id = $request->get('FileId');
    if (isset($project_id) && isset($file_id)) {
      // Get mappings between the job items and the file IDs, for the project.
      $remotes = RemoteMapping::loadByRemoteIdentifier('tmgmt_thebigword', $project_id);
      if (empty($remotes)) {
        \Drupal::logger('tmgmt_thebigword')->warning('Project %id not found.', ['%id' => $project_id]);
        return new Response(new FormattableMarkup('Project %id not found.', ['%id' => $project_id]), 404);
      }
      $remote = NULL;
      /** @var \Drupal\tmgmt\Entity\RemoteMapping $remote_candidate */
      foreach ($remotes as $remote_candidate) {
        if ($remote_candidate->getRemoteIdentifier3() == $file_id) {
          $remote = $remote_candidate;
        }
      }
      if (!$remote) {
        \Drupal::logger('tmgmt_thebigword')->warning('File %id not found.', ['%id' => $file_id]);
        return new Response(new FormattableMarkup('File %id not found.', ['%id' => $file_id]), 404);
      }

      /** @var \Drupal\tmgmt_thebigword\Plugin\tmgmt\Translator\ThebigwordTranslator $translator_plugin */
      $translator_plugin = $remote->getJob()->getTranslator()->getPlugin();
      $translator_plugin->setTranslator($remote->getJob()->getTranslator());

      $info = $translator_plugin->request('file/cmsstate/' . $file_id);
      if ($info['CmsState'] != $request->get('CmsState')) {
        \Drupal::logger('tmgmt_thebigword')->warning('The CmsState %cmsState is not the CmsState of the file %id.', ['%cmsState' => $info['CmsState']]);
        return new Response(new FormattableMarkup('The CmsState %cmsState is not the CmsState of the file %id.', ['%cmsState' => $info['CmsState']]), 400);
      }
      $job = $remote->getJob();
      $job_item = $remote->getJobItem();
      try {
        $translator_plugin->addFileDataToJob($remote->getJob(), $info['CmsState'], $project_id, $file_id);
      }
      catch (TMGMTException $e) {
        $restart_point = $info['CmsState'] == 'TranslatableReviewPreview' ? 'RestartPoint01' : 'RestartPoint02';
        $translator_plugin->sendFileError($restart_point, $project_id, $file_id, $job_item->getJob(), $remote->getRemoteData('RequiredBy'), $e->getMessage(), TRUE);
        $job->addMessage('Error fetching the job item: @job_item.', ['@job_item' => $job_item->label()], 'error');
      }
    }
    else {
      return new Response('Bad request.', 400);
    }
    return new Response();
  }

  /**
   * Returns a no preview response.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request to handle.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response to return.
   */
  public function noPreview(Request $request) {
    return new Response('No preview url available for this file.');
  }

  /**
   * Pull all remote translations.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request to handle.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response to return.
   */
  public function pullAllRemoteTrenslations(Request $request) {
    $translators = Translator::loadMultiple();
    $items = [];
    $limit = 50;
    $operations = [];

    /** @var \Drupal\tmgmt\Entity\Translator $translator */
    foreach ($translators as $translator) {
      $translator_plugin = $translator->getPlugin();
      if ($translator_plugin instanceof ThebigwordTranslator) {
        $query = \Drupal::entityQuery('tmgmt_job')
          ->condition('translator', $translator->id());
        $jobs = $query->execute();
        $query = \Drupal::entityQuery('tmgmt_job_item')
          ->condition('tjid', $jobs, 'IN');
        $or = $query->orConditionGroup()
          ->condition('state', JobItemInterface::STATE_ACTIVE)
          ->condition('state', JobItemInterface::STATE_REVIEW);
        $query->condition($or);
        $items = array_merge($query->execute(), $items);
      }
    }

    $chunks = array_chunk($items, $limit);

    foreach ($chunks as $chunk) {
      $operations[] = [
        [self::class, 'pullRemoteTranslations'],
        [$chunk],
      ];
    }
    $batch = [
      'title' => t('Pulling translations'),
      'operations' => $operations,
      'finished' => 'tmgmt_thebigword_pull_translations_batch_finished',
    ];
    batch_set($batch);
    return batch_process(Url::fromRoute('view.tmgmt_translation_all_job_items.page_1'));
  }

  /**
   * Creates continuous job items for entity.
   *
   * Batch callback function.
   */
  public static function pullRemoteTranslations(array $items, &$context) {
    if (!isset($context['results']['translated'])) {
      $context['results']['translated'] = 0;
    }
    $translated = $context['results']['translated'];
    /** @var \Drupal\tmgmt\JobItemInterface[] $job_items */
    $job_items = JobItem::loadMultiple($items);
    foreach ($job_items as $item) {
      /** @var \Drupal\tmgmt_thebigword\Plugin\tmgmt\Translator\ThebigwordTranslator $translator_plugin */
      $translator_plugin = $item->getJob()->getTranslatorPlugin();
      $translated += $translator_plugin->pullRemoteTranslation($item);
    }
    $context['results']['translated'] = $translated;
  }

}
